//
//  SITLocationRequest.h
//  SitumSDK
//
//  Created by A Barros on 27/2/17.
//  Copyright © 2017 Situm. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "SITBeaconFilter.h"
#import "SITOutdoorLocationOptions.h"

/// :nodoc:
typedef NS_ENUM(int, SITLocationPriority){

    kSITHighAccuracy = 0,
    
    kSITBalancedPowerAccuracy,
    
    kSITLowPower
} __attribute__((deprecated("Deprecated")));

/**
 * The interval indicating how often positions will be uploaded to realtime
 */
typedef NS_ENUM(NSInteger, SITRealtimeUpdateInterval) {
    ///The realtime dashboard will update in real time.
    kSITUpdateIntervalRealtime = 1,
    /// The realtime dashboard will update faster, but not in real time.
    kSITUpdateIntervalFast = 5,
    /// A compromise between real time and low power consumption
    kSITUpdateIntervalNormal = 15,
    /// The realtime will update slower.
    kSITUpdateIntervalSlow = 25,
    /// It will save a lot more battery but locations won't be shown in the real time.
    kSITUpdateIntervalBatterySaver = 1800,
    /// Positions will never be uploaded to realtime
    kSITUpdateNever = 0
};

/**
 A data object that contains parameters for the location service (SITLocationManager)
 */
@interface SITLocationRequest : NSObject <NSCopying> 

#pragma mark - Initializers

/**
 Constructor.
 
 @param buildingId Unique identifier of the building in which you want to locate a user.
 
 @return Initialized SITLocationRequest object.
 */
- (instancetype _Nonnull) initWithBuildingId: (NSString* _Nonnull) buildingId;

#pragma mark - Properties

/**
 The time interval indicating how often the position should be uploaded to realtime.
 */
@property (nonatomic, readonly) SITRealtimeUpdateInterval realtimeUpdateInterval;


/**
 * Set if you want to use dead reckoning to get fast position updates using only the inertial sensors,
 * between the server position updates. This parameter is incompatible with interval. If interval is
 * set, dead reckoning will be deactivated for this request.
 */
@property (nonatomic, readwrite) BOOL useDeadReckoning;


/**
 * Set if you want to use Gps for positioning indoor.
 */
@property (nonatomic, readwrite) BOOL useGps;


/**
 Unique identifier of the building in which you want to locate a user. From SDK version 2.43.0, if this value is not set (it is nil or an empty string), the positioning will start in indoor-outdoor mode, regardless of the value of the deprecated property `useGlobalLocation`.
 */
@property (nonatomic, strong) NSString * _Nullable buildingID;


/**
 The thread where the location updates will be provided. If not provided, the main thread will be used.
 */
@property (nonatomic, strong) NSOperationQueue * _Nullable operationQueue;


/**
 Additional options to modify the internal operation of the Location provider (private usage only).
 */
@property (nonatomic, strong) NSDictionary * _Nullable options;

/**
 Custom UUIDs. The system will scan beacons with those uuids
 */
@property (nonatomic, strong) NSArray<SITBeaconFilter *> * _Nullable beaconFilters;

/**
 Set the desired interval for location updates, in milliseconds. This interval is inexact, the service will try to keep this rate in a "best effort" way.
 This parameter is incompatible with dead reckoning. If set, dead reckoning will be deactivated for this request.
 */
@property (nonatomic) int interval;

/**
 Minimum displacement between location updates
 */
@property (nonatomic) float smallestDisplacement;

/**
 Set if you want to use barometer data for positioning.
 */
@property (nonatomic, readwrite) BOOL useBarometer;

/**
 Object containing several options to configure the indoor/outdoor positioning.
 This parameter will only be read if useGlobalLocation is set to YES
 */
@property (nonatomic, strong, readwrite) SITOutdoorLocationOptions* _Nullable outdoorLocationOptions;

/**
 Boolean value indicating if the floorplan images and the POI categories images for the building should be download to cache while starting positioning.
 Set to YES to preload the images.
 Set to NO to skip that process.
 Default value is YES.
 */
@property (nonatomic, readwrite) BOOL preloadInfo;

#pragma mark - Methods

/**
Setter. This parameter is incompatible with interval. If interval is set, dead reckoning will be deactivated for this request.
 
 @param useDeadReckoning Boolean value indicating if deadReckoning should be used.
 */
- (void) setUseDeadReckoning: (BOOL) useDeadReckoning;

/**
 Setter.
 
 @param useGps Boolean value indicating if gps positioning should be used.
 */
- (void) setUseGps: (BOOL) useGps;

/**
 Setter.
 
 @param realtimeUpdateInterval SITRealtimeUpdateInterval value indicating how often the position should be upload to realtime.
 */
- (void) setRealtimeUpdateInterval: (SITRealtimeUpdateInterval) realtimeUpdateInterval;

/**
 Setter. This parameter is incompatible with dead reckoning. If set, dead reckoning will be deactivated for this request.
 
 @param interval Integer value (in milliseconds) with the desired time interval for location updates.
 */
- (void) setInterval: (int) interval;

/**
 Setter. This parameter is incompatible with dead reckoning. If set, dead reckoning will be deactivated for this request.
 
 Keep in mind that if you set a custom value for {@link #interval(int)} you may
 not receive updates at the desired interval if the other condition isn't met.
 
 If this value is set the Dead Reckoning will be disabled
 
 @param smallestDisplacement Float value (in meters) with the minimum displacement between location updates.
 */
- (void) setSmallestDisplacement: (float) displacement;

/**
 Setter.
 
 @param useBarometer Boolean value indicating if barometer should be used for positioning.
 */
- (void) setUseBarometer: (BOOL) useBarometer;


/**
 Check if the request is valid.

 @return BOOL value that indicates if the request is valid (YES) or not (NO).
 */
- (BOOL)isValid;

/**
Check if indoor-outdoor mode is being used. From SDK version 2.43.0, indoor-outdoor mode is activated if  location request, either the deprectated property `useGlobalLocation` is set to YES or `buildingID` is not set (it is nil or an empty string).
 
@return BOOL value that indicates if indoor-outdoor mode is being used (YES) or not (NO).
*/

-(BOOL)isUsingGlobalLocation;

#pragma mark - Deprecated

/// :nodoc:
typedef NS_ENUM(int, SITLocationProvider){
    /**
     *  Localization can use the best provider available at any given moment (default).
     */
    kSITHybridProvider = 0,
    
    /**
     *  Use only cloud provider.
     */
    kSITCloudProvider,
    
    /**
     Use only phone provider. (coming soon).
     */
    kSITInPhoneProvider
};

/**
 One of SITLocationProvider.
 */
@property (nonatomic, readwrite) SITLocationProvider provider __attribute__((deprecated("Location provider is no longer selectable. Provider will always be kSITInPhoneProvider")));


/**
 The time interval indicating how often the position should be uploaded to realtime. Deprecated, use realtimeUpdateInterval instead.
 */
@property (nonatomic, readwrite) NSInteger updateInterval __attribute__((deprecated("Use realtimeUpdateInterval instead")));

/// :nodoc:
@property (nonatomic, readwrite) SITLocationPriority priority __attribute__((deprecated("Deprecated")));

/**
 Set to true if you want to use indoor/outdoor positioning. Deprecated on SDK version 2.43.0. From that version on if `buildingID` is not set (it is nil or an empty string), indoor/outdoor positioning will be used, regardless of the value of this variable.
 
 @deprecated v2.43.0
 */
@property (nonatomic, readwrite) BOOL useGlobalLocation;
__attribute__((deprecated("Use buildingID instead, if not buildingID value is set, indoor/outdoor positioning will be activated")));

/**
 Setter.
 
 @param updateInterval Integer value indicating how often the position should be upload to realtime. Deprecated, use setRealtimeUpdateInterval instead.
 */
- (void) setUpdateInterval: (NSInteger) updateInterval __attribute__((deprecated("Use setRealtimeUpdateInterval instead")));

/**
 Constructor.
 
 @param priority one of SITLocationPriority
 @param provider one of SITLocationProvider
 @param updateInterval The time interval at which location will be uploaded to realtime
 @param buildingID unique identifier of the building in which you want to locate a user. (This property is mandatory).
 @param operationQueue the thread where the location updates will be provided. If not provided, the main thread will be used.
 @param options additional options to modify the internal operation of the Location provider (private usage only).
 @return initialized object.
 */
- (instancetype   _Nonnull)initWithPriority:(SITLocationPriority)priority
                                   provider:(SITLocationProvider)provider
                             updateInterval:(NSInteger)updateInterval
                                 buildingID:(NSString * _Nonnull)buildingID
                             operationQueue:(NSOperationQueue * _Nonnull)operationQueue
                                    options:(NSDictionary * _Nullable)options __attribute__((deprecated("Use base constructor instead")));

/**
 Constructor.
 
 @param priority one of SITLocationPriority
 @param provider one of SITLocationProvider
 @param updateInterval The time interval at which location will be uploaded to realtime
 @param buildingID unique identifier of the building in which you want to locate a user. (This property is mandatory).
 @param operationQueue the thread where the location updates will be provided. If not provided, the main thread will be used.
 @param useDeadReckoning determines if dead reckoning should be used
 @param options additional options to modify the internal operation of the Location provider (private usage only).
 @return initialized object.
 */
- (instancetype  _Nonnull)initWithPriority:(SITLocationPriority)priority
                                  provider:(SITLocationProvider)provider
                            updateInterval:(NSInteger)updateInterval
                                buildingID:(NSString * _Nonnull)buildingID
                            operationQueue:(NSOperationQueue * _Nonnull)operationQueue
                          useDeadReckoning:(BOOL)useDeadReckoning
                                   options:(NSDictionary * _Nullable)options __attribute__((deprecated("Use base constructor instead")));
/**
 Compares the receiving locationRequest to another locationRequest
 
 @param aLocationRequest SITLocationRequest The location Request to compare with the receiver
 
 @return YES if aLocationRequest is equivalent to the receiving locationRequest, otherwise NO.
 
 */
-(BOOL) isEqualToLocationRequest:(SITLocationRequest *_Nonnull)aLocationRequest;

@end
